;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Disc Respect, An Amstrad CPC Game 
;;  Copyright (C) 2019 
;;  Made with CPCtelera: An Amstrad CPC Game Engine
;;  Sergio Sebastián: ssebasa98@gmail.com
;;  Roque Bru: elroquemadridista@gmail.com
;;  Sergio Campos: okicampos@gmail.com
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------


.include "lib/man/entity.h.s"
;.include "cpctelera.h.s"

.module physics

screen_width = 80
screen_height = 200

physics_init::
    ret

physics_update::
    ld b, a

_update_loop:
    ld a, #screen_width - 7 ;; Metemos en a el ancho de la pantalla mas 1
    sub entity_w(ix) ;; Le restamos el ancho de la entidad
    ld c, a ;; Lo guardamos en c

    ld a, entity_x(ix) ;; Guardamos en a la posicion x
    add entity_vx(ix) ;; Y le sumamos la velocidad
    cp c ;; A - C
    jr nc, invalid_x ;; Si te sales de la pantalla (la resta da menos de 0) va a invalid_x

    cp #8
    jr c, invalid_x

valid_x:
    ld entity_x(ix), a ;; Si la posicion es valida se guarda
    jr endif_x
invalid_x:
    ld a, entity_vx(ix) ;; Se guarda la velocidad en a
    neg ;; Cambiamos la velocidad de signo para ir hacia el otro lado
    ld entity_vx(ix), a ;; Se guarda la nueva velocidad
endif_x:

;; Compruebo si es enemy
    ld a, entity_type(ix)
    cp #1 ;; A - C
    jr z, esEnemy ;;Igual a 0

;; No es enemy
    ld a, #screen_height - 9
    sub entity_h(ix)
    ld c, a
    jr noEsEnemy

esEnemy:
    ld c, #0x54

noEsEnemy:
    ld a, entity_y(ix)
    add entity_vy(ix)
    cp c
    jr nc, invalid_y

;;Compruebo si es player
    ld a, entity_type(ix)
    cp #0x01 ;; A - C
    jr c, esPlayer ;; Igual a -1

;;No es player
    ld c, #10
    jr noEsPlayer

esPlayer:
    ld c, #0x65

noEsPlayer:
    ld a, entity_y(ix)
    add entity_vy(ix)
    cp c
    jr c, invalid_y

valid_y:
    ld entity_y(ix), a
    jr endif_y
invalid_y:
    ld a, entity_vy(ix)
    neg
    ld entity_vy(ix), a
endif_y:

    dec b ;; Decrementamos el numero de entidades
    ret z ;; Si el numero de entidades es 0, terminamos

    ld a, entity_type(ix) ;; Comprobamos si es un enemigo
    cp #1 ;; A - C
    jr z, esEnemigo ;;Igual a 0

volverAqui:
    ld de, #sizeof_e ;; Si no es 0, almacenamos el tamaño de la nueva entidad
    add ix, de
    jr _update_loop ;; Y repetimos

esEnemigo:
    ld a, entity_points(iy)
    cp #1
    jr c, volverAqui

physics_follow_ball:
    ld a, entity_type(iy)
    cp #2
    jr z, follow_ball
    ld a, entity_points(iy)
    cp #1
    jr z, volverAqui ;; Si es el segundo enemigo, solo persigue la bola
    ld a, entity_ball(ix)
    cp #0
    jr z, physics_defensivePositioning
    jr volverAqui

follow_ball: ;; Calcula si la pelota va hacia el enemigo
    ld a, #0xF0
    cp entity_vy(iy) ;; A - C = 0 - (-1) o 0 - 1
    jr nc, noVaHaciaEnemy ;; Si la vy de ball es negativa

cogeLaBola: ;; La pelota va hacia el enemigo
    ld a, entity_x(ix)
    cp entity_x(iy)
    jr z, noTeMuevas
    jr c, veALaDer

veALaIzq:
    ld entity_vx(ix), #0xFF
    jr volverAqui

veALaDer:
    ld entity_vx(ix), #01
    jr volverAqui

noTeMuevas:
    ld entity_vx(ix), #0
    jr volverAqui

noVaHaciaEnemy: ;; La pelota no va hacia el enemigo
    ld a, #40 ;; Si marcan a enemy tiene q coger la ball
    cp entity_y(iy)
    jr z, cogeLaBola
    jr noTeMuevas

physics_defensivePositioning:
    ld c, #0x24
    ld a, entity_x(iy)
    cp c
    jr c, playerEstaIzq
    jr nc, playerEstaDer
    jr irAlPuntoMedio

playerEstaIzq:
    ld c, entity_x(iy)
    ld a, #0x24

playerEstaDer:
    sub c ;; A = A - C
    srl a ;; Divido entre 2 corriendo la coma
    add c ;; Le sumo la posicion del jugador para obtener el punto medio
    ld c, a
    jr irAlPuntoMedio

irAlPuntoMedio:
    ld a, entity_x(ix)
    cp c
    jr z, noTeMuevas
    jr c, veALaDer
    jr veALaIzq