;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Disc Respect, An Amstrad CPC Game 
;;  Copyright (C) 2019 
;;  Made with CPCtelera: An Amstrad CPC Game Engine
;;  Sergio Sebastián: ssebasa98@gmail.com
;;  Roque Bru: elroquemadridista@gmail.com
;;  Sergio Campos: okicampos@gmail.com
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------


;;
;;  Render system
;;

.include "lib/sys/render.h.s"
.include "lib/man/entity.h.s"
.include "lib/assets/assets.h.s"

.module render

render_init::
    ld hl, #_bg_escenario
    ld de, #0xC000
    ld bc, #0x4000
    ldir

    ld hl, #0x030F ;; Eligo color de fondo 03 y color de letras 0F
    call cpct_setDrawCharM0_asm ;; Aplico esos colores
    _string0: .asciz "0" ;; Declaro un char
    ld iy, #_string0 ;; Lo meto en IY
    ld hl, #0xC052 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_string0 ;; Lo meto en IY
    ld hl, #0xC732 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    call entity_getBallIY
;
    _stringPRESS: .asciz "PRESS"
    _stringFIRE: .asciz "FIRE"
    _stringTO: .asciz "TO"
    _stringCONTINUE: .asciz "CONTINUE"
    _stringLOSE: .asciz "LOSEQQ" ;; Declaro un char
    _stringYOU: .asciz "YOU"
    _stringWIN: .asciz "WINQQ" ;; Declaro un char
    _stringCONGRATULATIONS: .asciz "CONGRATULATIONSQQ"
    _stringARE: .asciz "ARE"
    _stringTHE: .asciz "THE"
    _stringWORLD: .asciz "WORLD"
    _stringCHAMPION: .asciz "CHAMPIONQQQ"
    _stringCOPA: .asciz "VVVVV"
    
    ret

render_update::
    ld de, #0xD3C8
    ld a, #0xFF
    ld c, #0x140
    ld b, #2
    call cpct_drawSolidBox_asm ;; Dibuja la linea central

    call entity_getArrayIX
    call render_entities

    ret

render_entities::
    ld (_ent_counter), a

_update_loop:
    ;;TODO ESTO SERIA PARA REPINTAR EL FONDO
    ld e, entity_lastVP_l(ix) ;;ALmacena lo que habia antes de renderizar el sprite
    ld d, entity_lastVP_h(ix) ;;Last video memory pointer
    ld a, #0x33
    ld c, entity_w(ix)
    ld b, entity_h(ix)
    push bc
    call cpct_drawSolidBox_asm

    ld de, #0xC000
    ld c, entity_x(ix)
    ld b, entity_y(ix)
    call cpct_getScreenPtr_asm

    ld entity_lastVP_l(ix), l
    ld entity_lastVP_h(ix), h

    ex de, hl
    ld l, entity_spriteL(ix)
    ld h, entity_spriteH(ix)
    pop bc
    call cpct_drawSprite_asm

_ent_counter = .+1 ;; Te lleva al contador de entidades
    ld a, #0 ;; Contador de entidades
    dec a
    ret z

    ld (_ent_counter), a
    ld bc, #sizeof_e
    add ix, bc
    jr _update_loop

render_punto::
    ld hl, #0x030F ;; Eligo color de fondo 03 y color de letras 0F
    call cpct_setDrawCharM0_asm ;; Aplico esos colores
    xor a
    cp entity_type(ix)
    jr z, puntoDePlayer
    ld hl, #0xC732
    jr puntoDeEnemigo

    puntoDePlayer:
        ld hl, #0xC052

    puntoDeEnemigo:

    ld a, entity_points(ix)
    cp #2
    jr c, unPunto
    jr z, dosPuntos

    tresPuntos:
        _string3: .asciz "K" ;; Declaro un char
        ld iy, #_string3 ;; Lo meto en IY
        jr fin

    dosPuntos:
        _string2: .asciz "J" ;; Declaro un char
        ld iy, #_string2 ;; Lo meto en IY
        jr fin

    unPunto:
        _string1: .asciz "B" ;; Declaro un char
        ld iy, #_string1 ;; Lo meto en IY

    fin:
    call _myDrawStringM0 ;; Pinto en la pantalla
    call entity_getBallIY
    ret

render_pant::
    ld c, #0
    call cpct_setVideoMode_asm ;; Ponemos el modo de video a 0
    ld hl, #_pal_main ;; hl almacena la paleta
    ld de, #16 ;; Que es de tamaño 16 (se almacena en de)
    call cpct_setPalette_asm
    ;cpctm_setBorder_asm HW_WHITE ;; Cambiamos el borde para ponerlo blanco (gris)

    ld hl, #_bg_principal
    ld de, #0xC000
    ld bc, #0x4000
    ldir

    loopWaitingEnter: ;TODO, hay que cambiar esto para evitar que si pulsa una de las teclas de control se interprete como control
    call cpct_scanKeyboard_asm
    ld hl, #Key_Enter
    call cpct_isKeyPressed_asm
    ret nz
    jr loopWaitingEnter

render_pant2:: ;; YOU LOSE!! PRESS FIRE TO CONTINUE
    ld a, #0xCC
    call draw_pintaFondoPantallaM0
    
    ld hl, #0x030F ;; Eligo color de fondo 03 y color de letras 0F
    call cpct_setDrawCharM0_asm ;; Aplico esos colores
    
    ld iy, #_stringYOU ;; Lo meto en IY
    ld hl, #0xD384 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringLOSE ;; Lo meto en IY
    ld hl, #0xD394 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_stringPRESS
    ld hl, #0xD690
    call _myDrawStringM0

    ld iy, #_stringFIRE
    ld hl, #0xD6A6
    call _myDrawStringM0

    ld iy, #_stringTO
    ld hl, #0xD6B7
    call _myDrawStringM0

    ld iy, #_stringCONTINUE
    ld hl, #0xD6C1
    call _myDrawStringM0

    call entity_getBallIY
    
    jp loopWaitingKey

render_pant3:: ;; YOU WIN!! PRESS FIRE TO CONTINUE
    ld a, #0x0C
    call draw_pintaFondoPantallaM0

    ld hl, #0x020F ;; Eligo color de fondo 03 y color de letras 0F
    call cpct_setDrawCharM0_asm ;; Aplico esos colores
    
    ld iy, #_stringYOU ;; Lo meto en IY
    ld hl, #0xD386 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringWIN ;; Lo meto en IY
    ld hl, #0xD396 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_stringPRESS
    ld hl, #0xD690
    call _myDrawStringM0

    ld iy, #_stringFIRE
    ld hl, #0xD6A6
    call _myDrawStringM0

    ld iy, #_stringTO
    ld hl, #0xD6B7
    call _myDrawStringM0

    ld iy, #_stringCONTINUE
    ld hl, #0xD6C1
    call _myDrawStringM0

    jp loopWaitingKey

render_pant4:: ;;CONGRATULATIONS!! YOU ARE THE WORLD CHAMPION!!! PRESS FIRE TO CONTINUE
    ld a, #0x0C
    call draw_pintaFondoPantallaM0

    ld hl, #0x020F ;; Eligo color de fondo 03 y color de letras 0F
    call cpct_setDrawCharM0_asm ;; Aplico esos colores
    
    ld iy, #_stringCONGRATULATIONS ;; Lo meto en IY
    ld hl, #0xD196 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringYOU ;; Lo meto en IY
    ld hl, #0xD377 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_stringARE ;; Lo meto en IY
    ld hl, #0xD387 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringTHE ;; Lo meto en IY
    ld hl, #0xD396 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_stringWORLD ;; Lo meto en IY
    ld hl, #0xD3A5 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringCHAMPION ;; Lo meto en IY
    ld hl, #0xD422 ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla
    
    ld iy, #_stringCOPA ;; Lo meto en IY
    ld hl, #0xD56E ;; Posicion de pantalla donde se mostrara
    call _myDrawStringM0 ;; Pinto en la pantalla

    ld iy, #_stringPRESS
    ld hl, #0xD690
    call _myDrawStringM0

    ld iy, #_stringFIRE
    ld hl, #0xD6A6
    call _myDrawStringM0

    ld iy, #_stringTO
    ld hl, #0xD6B7
    call _myDrawStringM0

    ld iy, #_stringCONTINUE
    ld hl, #0xD6C1
    call _myDrawStringM0

    call entity_getBallIY

    jr loopWaitingKey

draw_pintaFondoPantallaM0::
    ld de, #0xC000                
    ld c, #40                     
    ld b, #200                    
    push af       ;Si no metemos en pila, lo perdemos con el solidbox                
    call cpct_drawSolidBox_asm    
    pop af                        
    ld de, #0xC028                
    ld c, #40     ;Con solo iuna caja gurda no vale, hay que hacer 2               
    ld b, #200                    
    call cpct_drawSolidBox_asm    
    ret

loopWaitingKey: ;TODO, hay que cambiar esto para evitar que si pulsa una de las teclas de control se interprete como control
    call cpct_scanKeyboard_asm
    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    ret nz
    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    ret nz
    jr loopWaitingKey

;loopWaitingKey: ;TODO, hay que cambiar esto para evitar que si pulsa una de las teclas de control se interprete como control
;    call cpct_scanKeyboard_asm
;    ld hl, #Key_Space
;    call cpct_isKeyPressed_asm
;    ret nz
;    ld hl, #Joy0_Fire1
;    call cpct_isKeyPressed_asm
;    ret nz
;    dec entity_ball(iy)
;    jr nz, loopWaitingKey
;
;    ld a, l
;    cp #0x0F
;    jr z, gris
;    ld hl, #0x030F
;    jr blanco
;    gris:
;    ld hl, #0x030A
;    blanco:
;    call cpct_setDrawCharM0_asm
;    ld iy, #_stringPRESS
;    ld hl, #0xD6A0
;    call _myDrawStringM0
;
;    ld iy, #_stringFIRE
;    ld hl, #0xD6C0
;    call _myDrawStringM0
;    jr loopWaitingKey